@@ -2,15 +2,9 @@ from __future__ import absolute_import, division, print_function
 
 from _echopype_version import version as __version__  # noqa
 
-from . import calibrate, preprocess
+from . import calibrate, preprocess, utils
 from .convert.api import open_raw
 from .echodata.api import open_converted
 from .echodata.combine import combine_echodata
 
-__all__ = [
-    "open_raw",
-    "open_converted",
-    "combine_echodata",
-    "calibrate",
-    "preprocess",
-]
+__all__ = ["open_raw", "open_converted", "combine_echodata", "calibrate", "preprocess", "utils"]
