@@ -3,13 +3,14 @@ from typing import Optional, Union, List
 
 import xarray as xr
 
-from .plot import _plot_echogram, FacetGrid, QuadMesh, T
+from .plot import _plot_echogram, FacetGrid, QuadMesh
 from ..echodata import EchoData
 
 
 def create_echogram(
     data: Union[EchoData, xr.Dataset],
-    frequency: Union[int, float, List[T], None] = None,
+    channel: Union[str, List[str], None] = None,
+    frequency: Union[str, List[str], None] = None,
     get_range: Optional[bool] = None,
     range_kwargs: dict = {},
     water_level: Union[int, float, xr.DataArray, bool, None] = None,
@@ -21,17 +22,20 @@ def create_echogram(
     ----------
     data : EchoData or xr.Dataset
         Echodata or Xarray Dataset to be plotted
+    channel : str or list of str, optional
+        The channel to be plotted.
+        Otherwise all channels will be plotted.
     frequency : int, float, or list of float or ints, optional
         The frequency to be plotted.
-        Otherwise all frequency will be plotted.
+        If not specified, all frequency will be plotted.
     get_range : bool, optional
-        Flag as to whether range should be computed or not,
-        by default it will just plot range_bin as the yaxis.
+        Flag as to whether range (``echo_range``) should be computed or not,
+        by default it will just plot `range_sample`` as the yaxis.
 
         Note that for data that is "Sv" xarray dataset, `get_range` defaults
         to `True`.
     range_kwargs : dict
-        Keyword arguments dictionary for computing range.
+        Keyword arguments dictionary for computing range (``echo_range``).
         Keys are `env_params`, `waveform_mode`, and `encode_mode`.
     water_level : int, float, xr.DataArray, or bool, optional
         Water level data array for platform water level correction.
@@ -63,7 +67,14 @@ def create_echogram(
         'units': 'm',
     }
 
-    if isinstance(frequency, list) and len(frequency) == 1:
+    if channel and frequency:
+        warnings.warn(
+            "Both channel and frequency are specified. Channel filtering will be used."
+        )
+
+    if isinstance(channel, list) and len(channel) == 1:
+        channel = channel[0]
+    elif isinstance(frequency, list) and len(frequency) == 1:
         frequency = frequency[0]
 
     if isinstance(data, EchoData):
@@ -71,11 +82,13 @@ def create_echogram(
             raise ValueError(
                 "Visualization for AD2CP sonar model is currently unsupported."
             )
-        yaxis = 'range_bin'
+        yaxis = 'range_sample'
         variable = 'backscatter_r'
         ds = data.beam
+        if 'ping_time' in ds:
+            _check_ping_time(ds.ping_time)
         if get_range is True:
-            yaxis = 'range'
+            yaxis = 'echo_range'
 
             if data.sonar_model.lower() == 'azfp':
                 if 'azfp_cal_type' not in range_kwargs:
@@ -136,28 +149,28 @@ def create_echogram(
                     data_type=EchoData,
                     platform_data=data.platform,
                 )
-            ds = ds.assign_coords({'range': range_in_meter})
-            ds.range.attrs = range_attrs
+            ds = ds.assign_coords({'echo_range': range_in_meter})
+            ds.echo_range.attrs = range_attrs
 
     elif isinstance(data, xr.Dataset):
-        if 'ping_time' in data and data.ping_time.shape[0] < 2:
-            raise ValueError("Ping time must be greater or equal to 2 data points.")
+        if 'ping_time' in data:
+            _check_ping_time(data.ping_time)
         variable = 'Sv'
         ds = data
-        yaxis = 'range'
-        if 'range' not in data.dims and get_range is False:
+        yaxis = 'echo_range'
+        if 'echo_range' not in data.dims and get_range is False:
             # Range in dims indicates that data is MVBS.
-            yaxis = 'range_bin'
+            yaxis = 'range_sample'
 
         # If depth is available in ds, use it.
-        ds = ds.set_coords('range')
+        ds = ds.set_coords('echo_range')
         if water_level is not None:
-            ds['range'] = _add_water_level(
-                range_in_meter=ds.range,
+            ds['echo_range'] = _add_water_level(
+                range_in_meter=ds.echo_range,
                 water_level=water_level,
                 data_type=xr.Dataset,
             )
-        ds.range.attrs = range_attrs
+        ds.echo_range.attrs = range_attrs
     else:
         raise ValueError(f"Unsupported data type: {type(data)}")
 
@@ -166,17 +179,24 @@ def create_echogram(
         xaxis='ping_time',
         yaxis=yaxis,
         variable=variable,
+        channel=channel,
         frequency=frequency,
         **kwargs,
     )
 
 
+def _check_ping_time(ping_time):
+    if ping_time.shape[0] < 2:
+        raise ValueError("Ping time must have a length that is greater or equal to 2")
+
+
 def _add_water_level(
     range_in_meter: xr.DataArray,
     water_level: Union[int, float, xr.DataArray, bool],
     data_type: str,
     platform_data: Optional[xr.Dataset] = None,
 ) -> xr.DataArray:
+    # Below, we rename time3 to ping_time because range_in_meter is in ping_time
     if isinstance(water_level, bool):
         if water_level is True:
             if data_type == xr.Dataset:
@@ -189,7 +209,7 @@ def _add_water_level(
                     isinstance(platform_data, xr.Dataset)
                     and 'water_level' in platform_data
                 ):
-                    return range_in_meter + platform_data.water_level
+                    return range_in_meter + platform_data.water_level.rename({'time3': 'ping_time'})
                 else:
                     warnings.warn(
                         "Boolean type found for water level. Please provide platform data with water level in it or provide a separate water level data."  # noqa
@@ -199,6 +219,9 @@ def _add_water_level(
         return range_in_meter
     if isinstance(water_level, xr.DataArray):
         check_dims = range_in_meter.dims
+        if 'time3' in water_level:
+            water_level = water_level.rename({'time3': 'ping_time'})
+
         if not any(
             True if d in water_level.dims else False for d in check_dims
         ):
